/* This file is meant to be used with the SCILAB arduino  
 *   toolbox, however, it can be used from the IDE environment
 *  (or any other serial terminal) by typing commands like:
 *  
 *  Conversion ascii -> number
 *  48->'0' ... 57->'9' 58->':' 59->';' 60->'<' 61->'=' 62->'>' 63->'?' 64->'@' 
 *  65->'A' ... 90->'Z' 91->'[' 92->'\' 93->']' 94->'^' 95->'_' 96->'`' 
 *  97->'a' ... 122->'z'
 *  Digital read or write
 *  Dan0 or Dan1 : attach digital pin n (ascii from 2 to b) to input (0) or output (1)
 *  Drn : read digital value (0 or 1) on pin n (ascii from 2 to b)
 *  Dwn0 or Dwn1 : write 1 or 0 on pin n
 *  
 *  R3: send version of the server code
 *  Stepping motor:
 *  Sanp: attach digital pin n to pulse p
 *  Sm:   move
 */
#define DEBUG 1
#define RX_PIN 11
#define TX_PIN 10

// include the SoftwareSerial library so you can use its functions:
#ifdef DEBUG
#include <SoftwareSerial.h>
#endif

#ifndef LED_BUILTIN
#define LED_BUILTIN 2
#endif

char deviceType = 0;   // type of Arduino sensor / actuator to talk to
char cmd = 0;          // command to be executed on the sensor / actuator
char inOrOut = 0;      // input or output
char pinNum = 0;       // pin number to which the sensor is connected
char pulseNum = 0;     // stepping motor pulse number
char sensorData = 0;   // data to be written to the sensor / actuator
/*
 * signal table for single step forward
 */
byte pulseTable[]={0x01,0x02,0x04,0x08};
byte pulseTableIndex,pulseMask;
byte pinAssignment[4];

#ifdef DEBUG
SoftwareSerial serial1 = SoftwareSerial(RX_PIN,TX_PIN);
#endif

void setup() {
  // put your setup code here, to run once:
  pinMode(LED_BUILTIN, OUTPUT);
  Serial.begin(115200);
  Serial.flush();
  

#ifdef DEBUG  
  pinMode(RX_PIN, INPUT);
  pinMode(TX_PIN, OUTPUT);
  serial1.begin(115200);
  serial1.println("Scilab server is ready!"); 
#endif
  pulseTableIndex = 0;
}

void loop() {
  int i,pin,pulse;
  char rCode;
    
  if (Serial.available() > 0) {
    // read the incoming byte:
    deviceType = Serial.read();
    
#ifdef DEBUG    
    serial1.print(deviceType);
#endif
        
    switch (deviceType) {
      /* here for analogue reference and server version */
/*-------------------------------------------------------------------------------------
 * Analog reference and server version number
 */
    case 'R':
      while (!Serial.available());                   // wait for next character
      rCode = Serial.read();
#ifdef DEBUG      
      serial1.println(rCode);
#endif
      switch (rCode) {
      case '3':
      	serial1.println("Version 0.1");
	      Serial.print("V01");
      	Serial.flush();
	      break;
      default:
#ifdef DEBUG
        if (rCode != '\n')
      	  serial1.println("Unknown R code");
#endif        
	      break;
      }
      break;

 /*-------------------------------------------------------------------------------------
 * Digital read/write
 */                	  
      /* here for GPIO pin */
    case 'D':
      while (!Serial.available());                   // wait for next character
      cmd = Serial.read();
#ifdef DEBUG      
      serial1.print(cmd);
#endif	
      switch (cmd) {
	    /* here for "attach" */
      case 'a':
	      while (!Serial.available());                   // wait for next character
	      pinNum = Serial.read();
#ifdef DEBUG       
	      serial1.print(pinNum);
#endif
	      pin = pinNum-'0';
	      while (!Serial.available());                   // wait for next character
	        inOrOut = Serial.read();
#ifdef DEBUG
	        serial1.println(inOrOut);
#endif
	      /* here for input or output */
	      switch (inOrOut) {
	        /* input */
	      case '0':
	        pinMode(pin,INPUT);
#ifdef DEBUG         
	        serial1.print("Pin ");
	        serial1.print(pin);
	        serial1.println(" set to input mode"); 
#endif
	        break;
	      case '1':
	        pinMode(pin,OUTPUT);
#ifdef DEBUG
	        serial1.print("Pin ");
	        serial1.print(pin);
	        serial1.println(" set to ouput mode");            
#endif
	        break;
	      default:
#ifdef DEBUG
          if (inOrOut != '\n')
	          serial1.println("Unknown in/out mode should be 0 or 1");
#endif
	      }
	      break;
          case 'r':
	      /* here to read the pin */
	        while (!Serial.available());                   // wait for next character	  
#ifdef DEBUG
	        pinNum = Serial.read();
#endif
	        serial1.println(pinNum);
	        pin = pinNum-'0';
	        sensorData = digitalRead(pin);
#ifdef DEBUG
	        serial1.print("Read from pin ");
	        serial1.print(pin);
	        serial1.print(": ");
	        serial1.println(sensorData);
#endif          
	      break;
        
      case 'w':
    	/* here to write the pin */
	        while (!Serial.available());                   // wait for next character
	        pinNum = Serial.read();
#ifdef DEBUG
	        serial1.print(pinNum);
#endif
	        pin = pinNum-'0'; 
	        while (!Serial.available());                   // wait for next character
	        sensorData = Serial.read();
#ifdef DEBUG
	        serial1.println(sensorData);
#endif
	        switch(sensorData) {
	        case '0':
#ifdef DEBUG
            serial1.print("Writing 0 to GPIO pin ");
            serial1.println(pin);
#endif
	          digitalWrite(pin,LOW);
	          break;
	        case '1':
#ifdef DEBUG
            serial1.print("Writing 1 to GPIO pin ");
            serial1.println(pin);        
#endif
	          digitalWrite(pin,HIGH);
	          break;
	        default:
#ifdef DEBUG
            if (sensorData != '\n')
	            serial1.println("Unknown digital sensor data, should be 0 or 1");
#endif
	        }
	        break;
        default:
#ifdef DEBUG
          if (cmd != '\n')
            serial1.println("\nUnknown command, should be a: attach, r: read or w: write");
#endif
      }
      break;
      default:
#ifdef DEBUG
        if (deviceType != '\n')
          serial1.println("\nUnknown device, should be R: reference or D: digital read/write");
#endif
    }
  }
}
