/*
 * program to test the scilib serial interface 
 * copyright U. Raich Jan. 2019
 * This program sends counter values to the multi-function board
 * The program is released under the GNU Public License
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "arduino_serial.h"

int main(void) {
  int handle=-1;
  int errCode=-1;
  int port=0;
  int baudrate=115200;
  char ledNum;
  char onOff[5];
  char *version="V";
  char attach[5];
  char buf[10];
  int noOfCharsToRead=3;
  int versionSize = strlen(version);
  int onOffSize;
  int attachSize;;
  int i,ledCnt;
  int counter,ledMask;
  
  printf("Testing the Arduino SciLab server using serialLib\n");
    
  open_serial(&handle, &port, &baudrate,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not open the serial port /dev/ttyUSB%d\n",port);
  else
    printf("/dev/ttyUSB%d was successfully opened\n",port);
  write_serial(&handle,version,&versionSize,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not write cmd to Arduino\n");
  sleep(2);
  
  read_serial(&handle,buf,&noOfCharsToRead,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not read from Arduino\n");
  else {
    if (buf[0] != 'V') {
      printf("Could not read server version number\n");
      close_serial(&handle,&errCode);
    }
    printf("Version: %c.%c\n",buf[1],buf[2]);
  }
  /*
   * set pins 10..13 to output
   */
  for (i=0;i<4;i++) {
    ledNum='0' + 10+i;
    sprintf(attach,"%s%c%s","Da",ledNum,"1");
    printf("command sent: %s\n",attach);
    attachSize = strlen(attach);
    write_serial(&handle,attach,&attachSize,&errCode);
  }

  counter = 0;
  for (i=0;i<16;i++) {
    printf("Count: 0x%02x\n",counter);
    ledMask = 1;
    for (ledCnt=0;ledCnt<4;ledCnt++) {
      ledNum='0' + 10 + ledCnt;
      if (counter &ledMask)
	sprintf(onOff,"%s%c%s","Dw",ledNum,"0");
      else
	sprintf(onOff,"%s%c%s","Dw",ledNum,"1");
      onOffSize = strlen(onOff);
      write_serial(&handle,onOff,&onOffSize,&errCode);
      ledMask <<= 1;
      usleep(10000);
    }
    usleep(500000);
    counter++;
  }  
     
  for (ledCnt=0;ledCnt<4;ledCnt++) {
    ledNum='0' + 10 + ledCnt;
    sprintf(onOff,"%s%c%s","Dw",ledNum,"1");
    write_serial(&handle,onOff,&onOffSize,&errCode);
    usleep(10000);
  }
  usleep(500000);
  close_serial(&handle,&errCode);
  return 0;
}
