/* This file is meant to be used with the SCILAB arduino  
 *   toolbox, however, it can be used from the IDE environment
 *  (or any other serial terminal) by typing commands like:
 *  
 *  Conversion ascii -> number
 *  48->'0' ... 57->'9' 58->':' 59->';' 60->'<' 61->'=' 62->'>' 63->'?' 64->'@' 
 *  65->'A' ... 90->'Z' 91->'[' 92->'\' 93->']' 94->'^' 95->'_' 96->'`' 
 *  97->'a' ... 122->'z'
 *  Digital read or write
 *  Dan0 or Dan1 : attach digital pin n (ascii from 2 to b) to input (0) or output (1)
 *  Drn : read digital value (0 or 1) on pin n (ascii from 2 to b)
 *  Dwn0 or Dwn1 : write 1 or 0 on pin n
 *  
 *  R3: send version of the server code
 *  Stepping motor:
 *  Sanp: attach digital pin n to pulse p
 *  Sm:   move
 */
#define DEBUG 1
#define MULTI_FUNCTION 1

// include the SoftwareSerial library so you can use its functions:
#ifdef DEBUG
#include <SoftwareSerial.h>

#define RX_PIN 9
#define TX_PIN 8
#endif

#ifndef LED_BUILTIN
#define LED_BUILTIN 2
#endif

#ifdef MULTI_FUNCTION
int latchPin = 4;
int clockPin = 7;
int dataPin  = 8; // pins connected to the shift registers
int LED1 =13;

unsigned char clearCode   = 0xff;
unsigned char Dis_buf[]   = {0xF1,0xF2,0xF4,0xF8};
unsigned char cnt = 0;
unsigned char digit[4];

void clearDisplay() {
  pinMode(LED1,OUTPUT);
  pinMode(latchPin,OUTPUT);
  pinMode(clockPin,OUTPUT);
  pinMode(dataPin,OUTPUT);                                       // set the 7-segment control pins to output
  for(int i=0; i<=3; i++)                                        // loop over the 4 digits
  {
    digitalWrite(latchPin,LOW);                                  // don't latch
    shiftOut(dataPin,clockPin,MSBFIRST,clearCode);               // address the digt
    shiftOut(dataPin,clockPin,MSBFIRST,Dis_buf[i] );             // the LED code for the digit
    digitalWrite(latchPin,HIGH);                                 // latch the data   
    delay(2); 
  }
  delay(500);
  digitalWrite(LED1,HIGH);
}
#endif

char deviceType = 0;   // type of Arduino sensor / actuator to talk to

/*
 * signal table for single step forward
 */
byte pulseTable[]={0x01,0x02,0x04,0x08};
byte pulseTableIndex,pulseMask;
byte pinAssignment[4];

#ifdef DEBUG
SoftwareSerial serial1 = SoftwareSerial(RX_PIN,TX_PIN);
#endif

/*
 * the version command
 */
void serverVersion() {
#ifdef DEBUG  
    serial1.println("\nVersion 0.3");
#endif
    Serial.print("V03");
    Serial.flush();
}

/*
 * the digital read or write commands
 */
void digitalReadWrite() {
  int i,pin;
  char cmd;
  char sensorData = 0;   // data to be written to the sensor / actuator
  char pinNum = 0;       // pin number to which the sensor is connected
  char inOrOut = 0;      // input or output 
 
  while (!Serial.available());                   // wait for next character
  cmd = Serial.read();
#ifdef DEBUG      
  serial1.print(cmd);
#endif	
  switch (cmd) {
    /*------------------------------------------------------------------------ 
     * here for "attach" 
     *-----------------------------------------------------------------------*/
  case 'a':
    while (!Serial.available());                   // wait for next character
    pinNum = Serial.read();
#ifdef DEBUG       
    serial1.print(pinNum);
#endif
    pin = pinNum-'0';
    while (!Serial.available());                   // wait for next character
    inOrOut = Serial.read();
#ifdef DEBUG
    serial1.println(inOrOut);
#endif
    /* here for input or output */
    switch (inOrOut) {
      /* input */
    case '0':
      pinMode(pin,INPUT);
#ifdef DEBUG         
      serial1.print("Pin ");
      serial1.print(pin);
      serial1.println(" set to input mode"); 
#endif
      break;
    case '1':
      pinMode(pin,OUTPUT);
#ifdef DEBUG
      serial1.print("Pin ");
      serial1.print(pin);
      serial1.println(" set to ouput mode");            
#endif
      break;
    default:
#ifdef DEBUG
      if (inOrOut != '\n')
	serial1.println("Unknown in/out mode should be 0 or 1");
#endif
      break;
    }
    break;
  case 'r':
    /*------------------------------------------------------------------
     * here to read the pin 
     *-----------------------------------------------------------------*/
    while (!Serial.available());                   // wait for next character	  
    pinNum = Serial.read();
#ifdef DEBUG    
    serial1.println(pinNum);
#endif
    pin = pinNum-'0';
    sensorData = digitalRead(pin);
#ifdef DEBUG
    serial1.print("Read from pin ");
    serial1.print(pin);
    serial1.print(": ");
    serial1.println(sensorData);
#endif          
    break;
        
  case 'w':
    /*-------------------------------------------------------------------- 
     * here to write the pin 
     *--------------------------------------------------------------------*/
    while (!Serial.available());                   // wait for next character
    pinNum = Serial.read();
#ifdef DEBUG
    serial1.print(pinNum);
#endif
    pin = pinNum-'0'; 
    while (!Serial.available());                   // wait for next character
    sensorData = Serial.read();
#ifdef DEBUG
    serial1.println(sensorData);
#endif
    switch(sensorData) {
    case '0':
#ifdef DEBUG
      serial1.print("Writing 0 to GPIO pin ");
      serial1.println(pin);
#endif
      digitalWrite(pin,LOW);
      break;
    case '1':
#ifdef DEBUG
      serial1.print("Writing 1 to GPIO pin ");
      serial1.println(pin);        
#endif
      digitalWrite(pin,HIGH);
      break;
    default:
#ifdef DEBUG
      if (sensorData != '\n')
	serial1.println("Unknown digital sensor data, should be 0 or 1");
#endif
      break;
    }
  }
}
/*
 * the stepping motor commands
 */
void stepper() {
  int i,pin,pulse;
  char cmd;
  char pinNum = 0;       // pin number to which the sensor is connected
  char pulseNum = 0;      // input or output 
  
/*------------------------------------------------------------------------------------ 
  *  Stepping motor
  */

  while (!Serial.available());                       // wait for next character
  cmd = Serial.read();
#ifdef DEBUG      
  serial1.print(cmd);
#endif
  switch (cmd) {
  case 'a':
    while (!Serial.available());                   // wait for next character
    pinNum = Serial.read();
#ifdef DEBUG      
    serial1.print(pinNum);
#endif           
    pin = pinNum-'0';
    while (!Serial.available());                   // wait for next character
    pulseNum = Serial.read();
#ifdef DEBUG      
    serial1.println(pulseNum);
#endif          
    pulse = pulseNum-'0';
    if ((pulse <1) || (pulse > 4)) {
#ifdef DEBUG
      serial1.println("Illegal pulse number must be 1..4");
#endif
      break;
    }
    else {
      pinAssignment[pulse-1] = pin;
      pinMode(pin,OUTPUT);
#ifdef DEBUG            
      serial1.print("Assigned pin ");
      serial1.print(pin);
      serial1.print(" to pulse ");
      serial1.println(pulse);      
#endif
    }
    break;
    /*
     * move by 1 step
     */
  case 'm':
    for (i=0;i<4;i++) {
      if ((1<<pulseTableIndex)&pulseTable[i]) 
	      digitalWrite(pinAssignment[i],HIGH);
      else 
	      digitalWrite(pinAssignment[i],LOW);
    }
#ifdef DEBUG    
    serial1.println();
#endif
    pulseTableIndex++;
    if (pulseTableIndex > 3)
      pulseTableIndex = 0;
    break; 
    /*
     * set all coil signals to low
     */
  case 'c':
    for (i=0;i<4;i++) 
      digitalWrite(pinAssignment[i],LOW); 
#ifdef DEBUG         
    serial1.println();
#endif
    break;
                
  default:
#ifdef DEBUG
    serial1.println("\nUnknown cmd should be a: attach or m: move or c: clear");
#endif  
    break;                          
  }
}

void setup() {
  // put your setup code here, to run once:
  pinMode(LED_BUILTIN, OUTPUT);
  Serial.begin(115200);
  Serial.flush();
  
#ifdef DEBUG  
  pinMode(RX_PIN, INPUT);
  pinMode(TX_PIN, OUTPUT);
  serial1.begin(115200);
  serial1.println("Scilab server is ready!"); 
#endif
  pulseTableIndex = 0;
#ifdef MULTI_FUNCTION
  clearDisplay();
#endif
}

void loop() {
  int i,pin,pulse;
  char rCode;
    
  if (Serial.available() > 0) {
    // read the incoming byte:
    deviceType = Serial.read();
    
#ifdef DEBUG    
    serial1.print(deviceType);
#endif
        
    switch (deviceType) {
    case 'V':
      serverVersion();
      break;
    case 'D':
      digitalReadWrite();
      break;
    case 'S':
      stepper();
      break;
    default:
#ifdef DEBUG
        if (deviceType != '\n')
          serial1.println("\nUnknown device, should be V: version or D: digital read/write or S: stepper");
#endif
      break;
    }
  }
}
