/*
 * program to test the scilib serial interface 
 * copyright U. Raich Jan. 2019
 * The program is released under the GNU Public License
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "arduino_serial.h"

int main(void) {
  int handle=-1;
  int errCode=-1;
  int port=0;
  int baudrate=115200;
  char switchNum;
  char *version="V";
  char attach[5];
  char readSwitch[5];
  char buf[10];
  int noOfCharsToRead=3;
  int versionSize = strlen(version);
  int attachSize,readSwitchSize;
  int charsToRead=1;
  
  printf("Testing the Arduino SciLab server using serialLib\n");
  printf("Test switch reading\n");   
  open_serial(&handle, &port, &baudrate,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not open the serial port /dev/ttyUSB%d\n",port);
  else
    printf("/dev/ttyUSB%d was successfully opened\n",port);
  write_serial(&handle,version,&versionSize,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not write cmd to Arduino\n");
  
  read_serial(&handle,buf,&noOfCharsToRead,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not read from Arduino\n");
  else {
    if (buf[0] != 'V') {
      printf("Could not read server version number\n");
      close_serial(&handle,&errCode);
    }
    printf("Version: %c.%c\n",buf[1],buf[2]);
  }
  /*
   * set GPIO 15 to input
   */
  switchNum='0'+15;
  sprintf(attach,"%s%c%s","Da",switchNum,"0");
  attachSize=strlen(attach);
  printf("Sending %s to attach\n",attach);
  write_serial(&handle,attach,&attachSize,&errCode);

  sprintf(readSwitch,"%s%c","Dr",switchNum);
  readSwitchSize=strlen(readSwitch);
  printf("Sending %s to read\n",readSwitch);
  write_serial(&handle,readSwitch,&readSwitchSize,&errCode);
  printf("Trying to read...\n");
  read_serial(&handle,buf,&charsToRead,&errCode);
  usleep(50000);
  while (true) {
    write_serial(&handle,readSwitch,&readSwitchSize,&errCode);
    read_serial(&handle,buf,&charsToRead,&errCode);
    if (errCode != ARDUINO_SUCCESS)
      goto exit;
    printf("%s",buf);
    usleep(50000);
  }
exit:
  close_serial(&handle,&errCode);
  return 0;
}
