/* This file is meant to be used with the SCILAB arduino  
 *   toolbox, however, it can be used from the IDE environment
 *  (or any other serial terminal) by typing commands like:
 *  
 *  Conversion ascii -> number
 *  48->'0' ... 57->'9' 58->':' 59->';' 60->'<' 61->'=' 62->'>' 63->'?' 64->'@' 
 *  65->'A' ... 90->'Z' 91->'[' 92->'\' 93->']' 94->'^' 95->'_' 96->'`' 
 *  97->'a' ... 122->'z'
 *  Digital read or write
 *  Dan0 or Dan1 : attach digital pin n (ascii from 2 to b) to input (0) or output (1)
 *  Drn : read digital value (0 or 1) on pin n (ascii from 2 to b)
 *  Dwn0 or Dwn1 : write 1 or 0 on pin n
 *  
 *  V: send version of the server code
 *  
 *  Stepping motor:
 *  Sanp: attach digital pin n to pulse p
 *  Sm:   move
 *  
 *  Analogue read:
 *  Ap: p: pin number 0..5
 *  
 * Analog write (PWM)
 * Ppv: write value v to pin p
 */
#define DEBUG 1
#define MULTI_FUNCTION 1

// include the SoftwareSerial library so you can use its functions:
#ifdef DEBUG
#include <SoftwareSerial.h>
#endif

#ifndef LED_BUILTIN
#define LED_BUILTIN 2
#endif

#ifdef MULTI_FUNCTION
#define RX_PIN 9
#define TX_PIN 2
int latchPin = 4;
int clockPin = 7;
int dataPin  = 8; // pins connected to the shift registers
int LED1 =13;

unsigned char clearCode   = 0xff;
unsigned char Dis_buf[]   = {0xF1,0xF2,0xF4,0xF8};
unsigned char Dis_table[] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0X80,0X90,0x88,0x83,0xc6,0xa1,0x86,0x8e};	    // LED coding for hex numbers
unsigned char digit[4];
int currentDigit=0;
bool displayActive=false;
long displayTrigger;

void clearDisplay() {
  pinMode(LED1,OUTPUT);
  pinMode(latchPin,OUTPUT);
  pinMode(clockPin,OUTPUT);
  pinMode(dataPin,OUTPUT);                                       // set the 7-segment control pins to output
  for(int i=0; i<=3; i++)                                        // loop over the 4 digits
  {
    digitalWrite(latchPin,LOW);                                  // don't latch
    shiftOut(dataPin,clockPin,MSBFIRST,clearCode);               // address the digt
    shiftOut(dataPin,clockPin,MSBFIRST,Dis_buf[i] );             // the LED code for the digit
    digitalWrite(latchPin,HIGH);                                 // latch the data   
    delay(2); 
  }
  delay(500);
  digitalWrite(LED1,HIGH);
}

/*
 * updates the seven segment display 
 * this should be done every 5 ms in order to get a stable, con flickering display
 */
void updateDisplay() {
  if (!displayActive)
    return;
  digitalWrite(latchPin,LOW);                                             // don't latch
                                                                          // shift in the 16 bits, most significant bit first
//  Serial.println(currentDigit);
  shiftOut(dataPin,clockPin,MSBFIRST,Dis_table[digit[currentDigit]]);     // address the digt
  shiftOut(dataPin,clockPin,MSBFIRST,Dis_buf[currentDigit] );             // the LED code for the digit
  digitalWrite(latchPin,HIGH);                                            // latch the data
  if (currentDigit == 3)
    currentDigit = 0;
  else
    currentDigit++;
}

#else
#define RX_PIN 11
#define TX_PIN 10
#endif
char deviceType = 0;   // type of Arduino sensor / actuator to talk to

/*
 * signal table for single step forward
 */
byte pulseTable[]={0x01,0x02,0x04,0x08};
byte pulseTableIndex,pulseMask;
byte pinAssignment[4];

#ifdef DEBUG
SoftwareSerial serial1 = SoftwareSerial(RX_PIN,TX_PIN);
#endif

void printHex16(uint16_t data) // prints 16-bit data in hex with leading zeroes
{
  uint8_t MSB=byte(data>>8);
  uint8_t LSB=byte(data);
         
  if (MSB<0x10) 
     Serial.print("0");
     Serial.print(MSB,HEX);
#ifdef DEBUG 
   if (MSB<0x10)   
     serial1.print("0");
     serial1.print(MSB,HEX);
#endif             
  if (LSB<0x10) 
     Serial.print("0"); 
     Serial.print(LSB,HEX);
#ifdef DEBUG 
   if (LSB<0x10)   
     serial1.print("0");
     serial1.print(LSB,HEX);
#endif                  
}

/*
 * the version command
 */
void serverVersion() {
#ifdef DEBUG  
    serial1.println("\nVersion 0.6");
#endif
    Serial.print("V06");
    Serial.flush();
}

#ifdef MULTI_FUNCTION
/*
 * seven segment Display
 */
void sevenSegment() {
  char cmd;
  char valueString[4];
  while (!Serial.available());                   // wait for next character
  cmd = Serial.read();
#ifdef DEBUG
  serial1.print(cmd);
#endif  
  switch (cmd) {
  /*
   * open: enable display
   */
    case 'o': 
      displayActive=true;
#ifdef DEBUG
      serial1.println();
#endif
      break;
    /*
     * close: disable display and clear it
     */
    case 'c':
      clearDisplay();
      displayActive=false;
#ifdef DEBUG
      serial1.println();
#endif
      break;
    case 'w':
      for (int i=0; i<4;i++) {
        while (!Serial.available());                   // wait for next character
        valueString[i] = Serial.read();
#ifdef DEBUG
        serial1.print(valueString[i]);
#endif              
      }
#ifdef DEBUG
      serial1.println();
#endif      
      for (int i=0;i<4;i++) {
        if ((valueString[i]>='0') && (valueString[i] <='9'))
          digit[i] = valueString[i]-'0';
        else if ((valueString[i] >= 'A') && (valueString[i] <='F'))
          digit[i] = valueString[i] -0x37;
        else if ((valueString[i] >= 'a') && (valueString[i] <='f'))
          digit[i] = valueString[i] -0x57;
        else {
#ifdef DEBUG
        serial1.print(valueString[i]);
        serial1.println(" is not a valid hex number");
#endif
        ;
        }
      }
        
      break;
    default:
#ifdef DEBUG
      serial1.print(cmd);
      serial1.println(" is an inknown command, should be o: open, c:close w: write");
#endif
    break;
  }
}
#endif

/*
 * the analog read
 */
void analogueRead() {
  char pinNum=0;
  int i,pin;
  int dataRead;
 
  while (!Serial.available());                   // wait for next character
  pinNum = Serial.read();
#ifdef DEBUG
  serial1.println(pinNum);
  serial1.print("Analog Read from pin ");
  serial1.println(pinNum);
#endif
  pin = pinNum-'0';
  if ((pin < 0) || (pin > 5)) {
#ifdef DEBUG
    serial1.print(pin);
    serial1.println(" is an illegal analogue pin number!");
#endif 
    return;
  }
  dataRead=analogRead(pin+A0);
#ifdef DEBUG
  serial1.print("Read analog value: ");
  serial1.println(dataRead);
#endif;
  printHex16(dataRead);
#ifdef DEBUG)
  serial1.println();
#endif  
  return;
}
/*
 * analog write (PWM)
 */
void analogueWrite() {
  char pinNum=0;
  char valueString[3];
  char *endPtr;
  int i,pin;
  int data;
 
  while (!Serial.available());                   // wait for next character
  pinNum = Serial.read();
#ifdef DEBUG
  serial1.print(pinNum);
#endif
  pin = pinNum-'0';
  switch (pin) {
    case 10:
    case 11:
    case 13:
      break;
    default:
#ifdef DEBUG
    serial1.println();
    serial1.print(pin);
    serial1.println(" is an illegal pwm pin number!");
#endif 
    return;
  }
   
  while (!Serial.available());                   // wait for next character
  valueString[0] = Serial.read();
  while (!Serial.available());                   // wait for next character
  valueString[1] = Serial.read();
  valueString[2]='\0';
  data=strtoul(valueString,&endPtr,16);
  #ifdef DEBUG
  serial1.println(valueString);
  serial1.print("Writing ");
  serial1.print(data,HEX);
  serial1.print(" to pin ");
  serial1.println(pin);
#endif
  analogWrite(pin,data);
  return;
}
/*
 * the digital read or write commands
 */
void digitalReadWrite() {
  int i,pin;
  char cmd;
  char sensorData = 0;   // data to be written to the sensor / actuator
  char pinNum = 0;       // pin number to which the sensor is connected
  char inOrOut = 0;      // input or output 
  int dataRead;
 
  while (!Serial.available());                   // wait for next character
  cmd = Serial.read();
#ifdef DEBUG      
  serial1.print(cmd);
#endif	
  switch (cmd) {
    /*------------------------------------------------------------------------ 
     * here for "attach" 
     *-----------------------------------------------------------------------*/
  case 'a':
    while (!Serial.available());                   // wait for next character
    pinNum = Serial.read();
#ifdef DEBUG       
    serial1.print(pinNum);
#endif
    pin = pinNum-'0';
    while (!Serial.available());                   // wait for next character
    inOrOut = Serial.read();
#ifdef DEBUG
    serial1.println(inOrOut);
#endif
    /* here for input or output */
    switch (inOrOut) {
      /* input */
    case '0':
      pinMode(pin,INPUT);
#ifdef DEBUG         
      serial1.print("Pin ");
      serial1.print(pin);
      serial1.println(" set to input mode"); 
#endif
      break;
    case '1':
      pinMode(pin,OUTPUT);
#ifdef DEBUG
      serial1.print("Pin ");
      serial1.print(pin);
      serial1.println(" set to ouput mode");            
#endif
      break;
    default:
#ifdef DEBUG
      if (inOrOut != '\n')
	serial1.println("Unknown in/out mode should be 0 or 1");
#endif
      break;
    }
    break;
  case 'r':
    /*------------------------------------------------------------------
     * here to read the pin 
     *-----------------------------------------------------------------*/
    while (!Serial.available());                   // wait for next character	  
    pinNum = Serial.read();
#ifdef DEBUG    
    serial1.println(pinNum);
#endif
    pin = pinNum-'0';
    dataRead = digitalRead(pin);
#ifdef DEBUG
    serial1.print("Read from pin ");
    serial1.print(pin);
    serial1.print(": ");
    serial1.println(dataRead);
#endif
    if(dataRead)
      Serial.print("1");   
    else 
      Serial.print("0");   
    Serial.flush();         
    break;
        
  case 'w':
    /*-------------------------------------------------------------------- 
     * here to write the pin 
     *--------------------------------------------------------------------*/
    while (!Serial.available());                   // wait for next character
    pinNum = Serial.read();
#ifdef DEBUG
    serial1.print(pinNum);
#endif
    pin = pinNum-'0'; 
    while (!Serial.available());                   // wait for next character
    sensorData = Serial.read();
#ifdef DEBUG
    serial1.println(sensorData);
#endif
    switch(sensorData) {
    case '0':
#ifdef DEBUG
      serial1.print("Writing 0 to GPIO pin ");
      serial1.println(pin);
#endif
      digitalWrite(pin,LOW);
      break;
    case '1':
#ifdef DEBUG
      serial1.print("Writing 1 to GPIO pin ");
      serial1.println(pin);        
#endif
      digitalWrite(pin,HIGH);
      break;
    default:
#ifdef DEBUG
      if (sensorData != '\n')
	serial1.println("Unknown digital sensor data, should be 0 or 1");
#endif
      break;
    }
  }
}
/*
 * the stepping motor commands
 */
void stepper() {
  int i,pin,pulse;
  char cmd;
  char pinNum = 0;       // pin number to which the sensor is connected
  char pulseNum = 0;      // input or output 
  
/*------------------------------------------------------------------------------------ 
  *  Stepping motor
  */

  while (!Serial.available());                       // wait for next character
  cmd = Serial.read();
#ifdef DEBUG      
  serial1.print(cmd);
#endif
  switch (cmd) {
  case 'a':
    while (!Serial.available());                   // wait for next character
    pinNum = Serial.read();
#ifdef DEBUG      
    serial1.print(pinNum);
#endif           
    pin = pinNum-'0';
    while (!Serial.available());                   // wait for next character
    pulseNum = Serial.read();
#ifdef DEBUG      
    serial1.println(pulseNum);
#endif          
    pulse = pulseNum-'0';
    if ((pulse <1) || (pulse > 4)) {
#ifdef DEBUG
      serial1.println("Illegal pulse number must be 1..4");
#endif
      break;
    }
    else {
      pinAssignment[pulse-1] = pin;
      pinMode(pin,OUTPUT);
#ifdef DEBUG            
      serial1.print("Assigned pin ");
      serial1.print(pin);
      serial1.print(" to pulse ");
      serial1.println(pulse);      
#endif
    }
    break;
    /*
     * move by 1 step
     */
  case 'm':
    for (i=0;i<4;i++) {
      if ((1<<pulseTableIndex)&pulseTable[i]) 
	      digitalWrite(pinAssignment[i],HIGH);
      else 
	      digitalWrite(pinAssignment[i],LOW);
    }
#ifdef DEBUG    
    serial1.println();
#endif
    pulseTableIndex++;
    if (pulseTableIndex > 3)
      pulseTableIndex = 0;
    break; 
    /*
     * set all coil signals to low
     */
  case 'c':
    for (i=0;i<4;i++) 
      digitalWrite(pinAssignment[i],LOW); 
#ifdef DEBUG         
    serial1.println();
#endif
    break;
                
  default:
#ifdef DEBUG
    serial1.println("\nUnknown cmd should be a: attach or m: move or c: clear");
#endif  
    break;                          
  }
}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);
  Serial.flush();
  
#ifdef DEBUG  
  pinMode(RX_PIN, INPUT);
  pinMode(TX_PIN, OUTPUT);
  serial1.begin(115200);
  serial1.println("Scilab server is ready!"); 
#endif
  pulseTableIndex = 0;
#ifdef MULTI_FUNCTION
  clearDisplay();
#endif
}

void loop() {
  int i,pin,pulse;
  char rCode;

#ifdef MULTI_FUNCTION
  updateDisplay();
#endif    
  if (Serial.available() > 0) {
    // read the incoming byte:
    deviceType = Serial.read();
    
#ifdef DEBUG    
    serial1.print(deviceType);
#endif
        
    switch (deviceType) {
    case 'V':
      serverVersion();
      break;
    case 'D':
      digitalReadWrite();
      break;
    case 'S':
      stepper();
      break;
    case 'A':
      analogueRead();
      break;
    case 'P':
      analogueWrite();
      break;
#ifdef MULTI_FUNCTION      
    case '7':
      sevenSegment();
      break;
#endif             
    default:
#ifdef DEBUG
        if (deviceType != '\n')
          serial1.println("\nUnknown device, should be V: version or D: digital read/write or S: stepper");
#endif
      break;
    }
  }
}
