/*
 * program to test the scilib serial interface 
 * copyright U. Raich Jan. 2019
 * The program is released under the GNU Public License
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "arduino_serial.h"

#define SERVO_PIN 7

int main(void) {
  int handle=-1;
  int errCode=-1;
  int port=0;
  int baudrate=115200;
  char *version="V";
  char attach[4];
  int  attachSize;
  char *detach="Od";
  int  detachSize=strlen(detach);
  char pinNum;
  int i;
  char angle[8];
  int  angleSize;

  char buf[10];
  int noOfVersionCharsToRead=3;
  int versionSize = strlen(version);
#ifdef READBACK  
  char readCmd[4];
  int  readCmdSize;
  int posCharsToRead=4;
#endif
  
  printf("Testing the Arduino SciLab server using serialLib\n");
  printf("Test servo motor\n");   
  open_serial(&handle, &port, &baudrate,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not open the serial port /dev/ttyUSB%d\n",port);
  else
    printf("/dev/ttyUSB%d was successfully opened\n",port);
  write_serial(&handle,version,&versionSize,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not write cmd to Arduino\n");
  
  read_serial(&handle,buf,&noOfVersionCharsToRead,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not read from Arduino\n");
  else {
    if (buf[0] != 'V') {
      printf("Could not read server version number\n");
      close_serial(&handle,&errCode);
    }
    printf("Version: %c.%c\n",buf[1],buf[2]);
  }

  /*
   * attach to the servo motor
   */
  pinNum=SERVO_PIN + '0';
  sprintf(attach,"Oa%c",pinNum); 
  attachSize=strlen(attach);
  printf("attachSize: %d string: %s\n",attachSize,attach);
  write_serial(&handle,attach,&attachSize,&errCode);
  /*
   * set the positive servo angle
   */
  angle[5]='\0';
  i=0;
  sprintf(angle,"Ow%c%04d",pinNum,i);
  angleSize=strlen(angle);
#ifdef READBACK
  sprintf(readCmd,"Or%c",pinNum);
  readCmdSize=strlen(readCmd);
#endif
  for (i=0;i<180;i++) {
    sprintf(angle,"Ow%c%04d",pinNum,i);
    printf("Written: %s, no of chars: %d\n",angle,angleSize);
    write_serial(&handle,angle,&angleSize,&errCode);
    usleep(15000);
#ifdef READBACK
    /*
     * read back the position 
     */  
    write_serial(&handle,readCmd,&readCmdSize,&errCode);
    read_serial(&handle,buf,&posCharsToRead,&errCode);
    buf[posCharsToRead]='\0';
    printf("Position read back: %s\n",buf);
#endif  
  }
  /*
   * set the negative servo angle
   */
  for (i=180;i>=0;i--) {
    sprintf(angle,"Ow%c%04d",pinNum,i);
    printf("Written: %s\n",angle);
    write_serial(&handle,angle,&angleSize,&errCode);
    usleep(15000);
#ifdef READBACK    
    /*
     * read back the position 
     */  
    write_serial(&handle,readCmd,&readCmdSize,&errCode);
    read_serial(&handle,buf,&posCharsToRead,&errCode);
    buf[posCharsToRead]='\0';
    printf("Position read back: %s\n",buf);
#endif
  }
  /*
   * detach the servo
   */
  write_serial(&handle,detach,&detachSize,&errCode);
  usleep(10000);
  close_serial(&handle,&errCode);
  return 0;
}
