/*
 * program to test the scilib serial interface 
 * copyright U. Raich Jan. 2019
 * The program is released under the GNU Public License
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <signal.h>
#include "arduino_serial.h"

int handle=-1;
int errCode=-1;

void exitProg(int dummy) {
  char *off="7c";
  int offSize=strlen(off);
  /*
    clean everything and exit 
  */
  printf("\nSwitching display off and exiting\n");
  write_serial(&handle,off,&offSize,&errCode);
  close_serial(&handle,&errCode);
  exit(1);
}

int main(void) {
  int port=0;
  int baudrate=115200;
  char *on ="7o";
  char *version="V";
  char valueCmd[7];
  int  writeCmdSize;
  char buf[10];
  int noOfCharsToRead=3;
  int versionSize = strlen(version);
  int onSize = strlen(on);
  uint16_t  counter=0;
  
  printf("Testing the Arduino SciLab server using serialLib\n");
  printf("Test for seven segment display\n");
  /*
    capture ^C to gracefully exit the program
  */
  signal(SIGINT,exitProg);
  
  open_serial(&handle, &port, &baudrate,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not open the serial port /dev/ttyUSB%d\n",port);
  else
    printf("/dev/ttyUSB%d was successfully opened\n",port);
  write_serial(&handle,version,&versionSize,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not write cmd to Arduino\n");
  
  read_serial(&handle,buf,&noOfCharsToRead,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not read from Arduino\n");
  else {
    if (buf[0] != 'V') {
      printf("Could not read server version number\n");
      close_serial(&handle,&errCode);
    }
    printf("Version: %c.%c\n",buf[1],buf[2]);
  }
  /*
   * switch display on
   */
  write_serial(&handle,on,&onSize,&errCode);
  usleep(50000);
  while (true) {
    sprintf(valueCmd,"7h%04x",counter);
    writeCmdSize=strlen(valueCmd);
    write_serial(&handle,valueCmd,&writeCmdSize,&errCode);
    counter++;
    usleep(200000);
  }

  return 0;
}
