/*
 * program to test the scilib serial interface 
 * copyright U. Raich Jan. 2019
 * The program is released under the GNU Public License
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "arduino_serial.h"

int main(void) {
  int handle=-1;
  int errCode=-1;
  int port=0;
  int baudrate=115200;
  char *version="V";
  char *readPot="A0";
  int readPotSize=strlen(readPot);
  char buf[10];
  int noOfVersionCharsToRead=3;
  int versionSize = strlen(version);
  int noOfAnalogCharsToRead=4;
  
  printf("Testing the Arduino SciLab server using serialLib\n");
  printf("Test switch reading\n");   
  open_serial(&handle, &port, &baudrate,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not open the serial port /dev/ttyUSB%d\n",port);
  else
    printf("/dev/ttyUSB%d was successfully opened\n",port);
  write_serial(&handle,version,&versionSize,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not write cmd to Arduino\n");
  
  read_serial(&handle,buf,&noOfVersionCharsToRead,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not read from Arduino\n");
  else {
    if (buf[0] != 'V') {
      printf("Could not read server version number\n");
      close_serial(&handle,&errCode);
    }
    printf("Version: %c.%c\n",buf[1],buf[2]);
  }

  while (true) {
    write_serial(&handle,readPot,&readPotSize,&errCode);
    read_serial(&handle,buf,&noOfAnalogCharsToRead,&errCode);
    if (errCode != ARDUINO_SUCCESS)
      goto exit;
    buf[noOfAnalogCharsToRead]='\0';
    printf("%s",buf);
    usleep(50000);
  }
exit:
  close_serial(&handle,&errCode);
  return 0;
}
