/*
 * program to test the scilib serial interface 
 * copyright U. Raich Jan. 2019
 * The program is released under the GNU Public License
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "arduino_serial.h"

#define A1 15

int main(void) {
  
  int  i,handle=-1;
  int  encoderValue;
  int  errCode=-1;
  int  port=0;
  int  clkPin=4, dtPin=5;
  int  baudrate=115200;
  char *version="V";
  char attach[5];
  int  attachSize;
  char detach[5];
  int  detachSize;
  char *getValue="Er";
  int  getValueSize=strlen(getValue);
  char buf[10];
  int  noOfVersionCharsToRead=3;
  int  versionSize = strlen(version);
  int  noOfEncoderCharsToRead=4;

  
  printf("Testing the Arduino SciLab server using serialLib\n");
  printf("Test switch reading\n");   
  open_serial(&handle, &port, &baudrate,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not open the serial port /dev/ttyUSB%d\n",port);
  else
    printf("/dev/ttyUSB%d was successfully opened\n",port);
  write_serial(&handle,version,&versionSize,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not write cmd to Arduino\n");
  
  read_serial(&handle,buf,&noOfVersionCharsToRead,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not read from Arduino\n");
  else {
    if (buf[0] != 'V') {
      printf("Could not read server version number\n");
      close_serial(&handle,&errCode);
    }
    printf("Version: %c.%c\n",buf[1],buf[2]);
  }

  sprintf(attach,"Ea%d%d",clkPin,dtPin);
  attachSize = strlen(attach);
  write_serial(&handle,attach,&attachSize,&errCode);
  printf("Sending: message %s of size %d\n",attach,attachSize);
  usleep(50000);
  sprintf(detach,"Ed%d%d",clkPin,dtPin);
  /*
   * read the encoder during ~ 5s at a frequency of 10 Hz
   * and print the value
   */
  for (i=0;i<50;i++) {
     write_serial(&handle,getValue,&getValueSize,&errCode);
     read_serial(&handle,buf,&noOfEncoderCharsToRead,&errCode);
     sscanf(buf,"%04x",&encoderValue);
     printf("value: %d\n",(int16_t)encoderValue);
     usleep(100000);
  }
  detachSize= strlen(detach);
  write_serial(&handle,detach,&detachSize,&errCode);
  printf("Sending: message %s of size %d\n",detach,detachSize);
  usleep(50000);  
  close_serial(&handle,&errCode);
  return 0;
}
