/*
 * program to test the scilib serial interface 
 * copyright U. Raich Jan. 2019
 * The program is released under the GNU Public License
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "arduino_serial.h"

int main(void) {
  int handle=-1;
  int errCode=-1;
  int port=0;
  int baudrate=115200;
  char *version="V";
  char attach[5];
  int attachSize=0;
  char *move="Sm";
  int moveSize=strlen(move);
  char *clear="Sc";
  int clearSize=strlen(clear);
  char buf[10];
  int noOfCharsToRead=3;
  int versionSize = strlen(version);
  int i;
  
  printf("Testing the Arduino SciLab server using serialLib\n");
    
  open_serial(&handle, &port, &baudrate,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not open the serial port /dev/ttyUSB%d\n",port);
  else
    printf("/dev/ttyUSB%d was successfully opened\n",port);
  write_serial(&handle,version,&versionSize,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not write cmd to Arduino\n");
  
  read_serial(&handle,buf,&noOfCharsToRead,&errCode);
  if (errCode != ARDUINO_SUCCESS)
    printf("Could not read from Arduino\n");
  else {
    if (buf[0] != 'V') {
      printf("Could not read server version number\n");
      close_serial(&handle,&errCode);
    }
    printf("Version: %c.%c\n",buf[1],buf[2]);
  }
  /*
   * associate pins 2..5 to pulse 0..3
   */
  for (i=0;i<4;i++) {
    sprintf(attach,"Sa%d%d",i+2,i+1);
    printf("Sending: %s to attach pin %d to pulse %d\n",attach,i+2,i+1);
    attachSize=strlen(attach);
    write_serial(&handle,attach,&attachSize,&errCode);
  }
  /*
   * move by 1000 steps
   */
  for (i=0;i<1000;i++) {
    write_serial(&handle,move,&moveSize,&errCode);
    usleep(5000);
  }
  /*
   * set all coil signals back to 0
   */
  write_serial(&handle,clear,&clearSize,&errCode);
  usleep(5000);
  close_serial(&handle,&errCode);
  return 0;
}
